"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = useBackground;
var _react = _interopRequireDefault(require("react"));
var _reactNative = require("react-native");
var _reactNativeSvg = require("react-native-svg");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// The assumed background styles are typed as followed:
//
// type Position = 'center' | 'left' | 'right' | 'top' | 'bottom';
// type PositionX = 'center' | 'left' | 'right';
// type PositionY = 'center' | 'top' | 'bottom';
//
// type BackgroundStyles = {
//   backgroundImage: string,
//   backgroundSize: 'contain' | 'cover' | string | number,
//   backgroundPosition: Position,
//   backgroundPositionX:
//     | PositionX
//     | { position: PositionX, offset: string | number },
//   backgroundPositionY:
//     | PositionY
//     | { position: PositionY, offset: string | number },
// };
const styles = _reactNative.StyleSheet.create({
  container: {
    overflow: 'hidden'
  },
  object: {
    aspectRatio: 1
  }
});
const horizontalPositions = {
  left: function () {
    let offset = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    return {
      alignItems: _reactNative.I18nManager.isRTL ? 'flex-end' : 'flex-start',
      paddingLeft: offset
    };
  },
  right: function () {
    let offset = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    return {
      alignItems: _reactNative.I18nManager.isRTL ? 'flex-start' : 'flex-end',
      paddingRight: offset
    };
  },
  center: offset => {
    if (offset !== undefined) {
      return null;
    }
    return {
      alignItems: 'center'
    };
  }
};
const verticalPositions = {
  top: function () {
    let offset = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    return {
      justifyContent: 'flex-start',
      paddingTop: offset
    };
  },
  bottom: function () {
    let offset = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    return {
      justifyContent: 'flex-end',
      paddingBottom: offset
    };
  },
  center: offset => {
    if (offset !== undefined) {
      return null;
    }
    return {
      justifyContent: 'center'
    };
  }
};
const getXml = value => {
  const match = value.match(/^url\("data:image\/svg\+xml,(.*?)"\)$/);
  if (!match) {
    return null;
  }
  return decodeURIComponent(match[1]);
};
const transforms = {
  backgroundSize(value) {
    if (value === 'cover') {
      return {
        width: '100%'
        // TODO: Support cover for views with height > width.
        // height: '100%',
      };
    }

    if (value === 'contain') {
      return {
        width: '100%',
        height: '100%'
      };
    }
    const {
      width,
      height = width
    } = value;
    return {
      width,
      height
    };
  },
  backgroundPosition(value) {
    if (value === 'center') {
      return {
        alignItems: 'center',
        justifyContent: 'center'
      };
    }
    if (horizontalPositions[value]) {
      return {
        ...horizontalPositions[value](),
        justifyContent: 'center'
      };
    }
    if (verticalPositions[value]) {
      return {
        alignItems: 'center',
        ...verticalPositions[value]()
      };
    }
    return null;
  },
  backgroundPositionX(value) {
    const {
      position = 'left',
      offset
    } = typeof value === 'object' ? value : {
      position: value
    };
    if (!horizontalPositions[position]) {
      return null;
    }
    return horizontalPositions[position](offset);
  },
  backgroundPositionY(value) {
    const {
      position = 'top',
      offset
    } = typeof value === 'object' ? value : {
      position: value
    };
    if (!verticalPositions[position]) {
      return null;
    }
    return verticalPositions[position](offset);
  }
};
function useBackground(style) {
  const flattenedStyle = _reactNative.StyleSheet.flatten(style);
  const background = {
    containerStyle: {},
    objectStyle: {},
    xml: null
  };
  Object.entries(flattenedStyle).forEach(_ref => {
    let [key, value] = _ref;
    if (!value) {
      return;
    }
    if (key === 'backgroundImage') {
      background.xml = getXml(value);
    } else if (key === 'backgroundSize') {
      Object.assign(background.objectStyle, transforms[key](value));
    } else if (key === 'backgroundPosition' || key === 'backgroundPositionX' || key === 'backgroundPositionY') {
      Object.assign(background.containerStyle, transforms[key](value));
    }
  });
  return {
    style: flattenedStyle,
    element: /*#__PURE__*/_react.default.createElement(_reactNative.View, {
      style: [_reactNative.StyleSheet.absoluteFill, styles.container, background.containerStyle]
    }, /*#__PURE__*/_react.default.createElement(_reactNative.View, {
      style: [styles.object, background.objectStyle]
    }, background.xml && /*#__PURE__*/_react.default.createElement(_reactNativeSvg.SvgXml, {
      xml: background.xml
    })))
  };
}
//# sourceMappingURL=useBackground.native.js.map