"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = transform;
var _reactNative = require("react-native");
var _cssToReactNative = require("css-to-react-native");
var _rem = _interopRequireDefault(require("./rem"));
var _formula = _interopRequireDefault(require("./formula"));
var _rgba = _interopRequireDefault(require("./rgba"));
var _backgroundSize = _interopRequireDefault(require("./properties/backgroundSize"));
var _backgroundPosition = _interopRequireDefault(require("./properties/backgroundPosition"));
var _backgroundPositionX = _interopRequireDefault(require("./properties/backgroundPositionX"));
var _backgroundPositionY = _interopRequireDefault(require("./properties/backgroundPositionY"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const urlRE = /^url\(.*?\)$/g;
const applyVariables = (variables, key, value) => typeof value === 'function' ? value(variables, key) : value;
const applyTransforms = result => {
  // Do not transform values that belong to an url().
  if (result.match(urlRE)) {
    return result;
  }
  return (0, _rgba.default)((0, _formula.default)((0, _rem.default)(result)));
};
const resolveValue = (value, definition, theme, options) => {
  const variables = options.preferTheme ? {
    ...definition.variables,
    ...theme
  } : {
    ...theme,
    ...definition.variables
  };

  // If there is only one part, we allow other results than strings as well.
  if (value.length === 1) {
    const result = applyVariables(variables, options.key, value[0]);
    return typeof result === 'string' ? applyTransforms(result) : result;
  }
  const stringifiedResult = value.reduce((previous, current) => `${previous}${applyVariables(variables, options.key, current)}`, '');
  return applyTransforms(stringifiedResult);
};
function transform(children, theme) {
  let options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  let scopes = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];
  let variables = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};
  let definitions = [{
    scopes,
    declarations: {},
    variables: {
      ...variables
    }
  }];
  children.forEach(child => {
    if (child.type === 'variable' || child.type === 'declaration') {
      // Transform value to react native compatible value.
      const value = resolveValue(child.value, definitions[0], theme, options);
      if (child.type === 'variable') {
        definitions[0].variables[child.name] = value;
      }

      // Only add value if value is not null.
      if (child.type === 'declaration' && value !== 'null') {
        if (child.name === 'box-shadow') {
          if (_reactNative.Platform.OS === 'web') {
            Object.assign(definitions[0].declarations, {
              boxShadow: value
            });
          }
        } else if (child.name === 'margin-vertical') {
          Object.assign(definitions[0].declarations, {
            ...(0, _cssToReactNative.getStylesForProperty)('marginTop', value),
            ...(0, _cssToReactNative.getStylesForProperty)('marginBottom', value)
          });
        } else if (child.name === 'margin-horizontal') {
          // Among other things workaround for react-native issue #350
          // https://github.com/facebook/react-native/issues/350#issuecomment-375238958
          Object.assign(definitions[0].declarations, {
            ...(0, _cssToReactNative.getStylesForProperty)('marginLeft', value),
            ...(0, _cssToReactNative.getStylesForProperty)('marginRight', value)
          });
        } else if (child.name === 'padding-vertical') {
          Object.assign(definitions[0].declarations, {
            ...(0, _cssToReactNative.getStylesForProperty)('paddingTop', value),
            ...(0, _cssToReactNative.getStylesForProperty)('paddingBottom', value)
          });
        } else if (child.name === 'padding-horizontal') {
          Object.assign(definitions[0].declarations, {
            ...(0, _cssToReactNative.getStylesForProperty)('paddingLeft', value),
            ...(0, _cssToReactNative.getStylesForProperty)('paddingRight', value)
          });
        } else if (child.name === 'background-size') {
          Object.assign(definitions[0].declarations, (0, _backgroundSize.default)(value));
        } else if (child.name === 'background-position') {
          Object.assign(definitions[0].declarations, (0, _backgroundPosition.default)(value));
        } else if (child.name === 'background-position-x') {
          Object.assign(definitions[0].declarations, (0, _backgroundPositionX.default)(value));
        } else if (child.name === 'background-position-y') {
          Object.assign(definitions[0].declarations, (0, _backgroundPositionY.default)(value));
        } else if (child.name === 'border-color' && value.split(' ').length === 1) {
          // Workaround for react-native issue #19981
          // https://github.com/facebook/react-native/issues/19981
          Object.assign(definitions[0].declarations, {
            borderColor: value
          });
        } else {
          // TODO: Pre-process css-to-react-native transformation, so that we only
          // need to insert theme variables here.
          Object.assign(definitions[0].declarations, (0, _cssToReactNative.getStylesForProperty)((0, _cssToReactNative.getPropertyName)(child.name), value));
        }
      }
    }
    if (child.type === 'block') {
      definitions = definitions.concat(transform(child.children, theme, options,
      // Add child scopes to current scopes.
      [...scopes, ...child.scopes],
      // Pass down variables, so that we can extend them.
      definitions[0].variables));
    }
  });

  // Filter empty definitions and return.
  return definitions.filter(definition => Object.keys(definition.declarations).length > 0 || Object.keys(definition.variables).length > 0);
}
//# sourceMappingURL=index.js.map